import processing.serial.*;

Serial myPort;  // Create object from Serial class
static String val;    // Data received from the serial port
int sensorFreq = 0; // Frequency of the sound
int[] freqHistory; // Array to store frequency history
int historyLength = 500; // Length of history to display
int graphWidth = 800; // Width of the graph
int graphHeight = 400; // Height of the graph
boolean soundDetectionOn = true; // Flag to control sound detection
boolean isBlackBackground = true; // Flag to control background color
Button soundButton, backgroundButton; // Buttons for sound detection and background color

void settings() {
  size(graphWidth, graphHeight);
}

void setup() {
  background(0);
  stroke(255);
  noFill();
  freqHistory = new int[historyLength];
  String portName = "COM11"; // Change the port name to match your setup
  myPort = new Serial(this, portName, 9600);
  
  // Initialize buttons
  soundButton = new Button(20, 20, 100, 40, "Sound");
  backgroundButton = new Button(140, 20, 200, 40, "Background");
}

void draw() {
  if (soundDetectionOn && myPort.available() > 0) {  // If sound detection is on and data is available from the serial port
    val = myPort.readStringUntil('\n'); 
    try {
      if (val != null) { // Check if val is not null before using it
        sensorFreq = Integer.valueOf(val.trim());
        updateHistory(sensorFreq);
      }
    } catch(Exception e) {
      println("Error reading serial data: " + e);
    }
  }  
  
  // Draw graph
  drawGraph();
  
  // Draw buttons
  drawButtons();
}

void updateHistory(int newFreq) {
  // Shift the frequency history array to the left
  for (int i = 0; i < historyLength - 1; i++) {
    freqHistory[i] = freqHistory[i + 1];
  }
  // Add the new frequency to the end of the history array
  freqHistory[historyLength - 1] = newFreq;
}

void drawGraph() {
  // Set background color
  if (isBlackBackground) {
    background(0);
  } else {
    background(0, 0, 255); // Blue background
  }
  
  // Draw the graph lines
  for (int i = 0; i < historyLength - 1; i++) {
    float x1 = map(i, 0, historyLength - 1, 0, width);
    float x2 = map(i + 1, 0, historyLength - 1, 0, width);
    float y1 = map(freqHistory[i], 0, 1023, height, 0);
    float y2 = map(freqHistory[i + 1], 0, 1023, height, 0);
    line(x1, y1, x2, y2);
  }
}

void drawButtons() {
  fill(255);
  rect(20, 20, 100, 40);
  rect(140, 20, 200, 40);
  fill(0);
  textSize(16);
  textAlign(CENTER, CENTER);
  text("Sound", 70, 40);
  text("Background", 240, 40);
}

void mousePressed() {
  // Check if the mouse is over the sound button
  if (mouseX >= soundButton.x && mouseX <= soundButton.x + soundButton.w &&
      mouseY >= soundButton.y && mouseY <= soundButton.y + soundButton.h) {
    soundDetectionOn = !soundDetectionOn; // Toggle sound detection
  }
                                     
  // Check if the mouse is over the background button
  if (mouseX >= backgroundButton.x && mouseX <= backgroundButton.x + backgroundButton.w &&
      mouseY >= backgroundButton.y && mouseY <= backgroundButton.y + backgroundButton.h) {
    isBlackBackground = !isBlackBackground; // Toggle background color
  }
}

class Button {
  float x, y, w, h;
  String label;
  
  Button(float x, float y, float w, float h, String label) {
    this.x = x;
    this.y = y;
    this.w = w;
    this.h = h;
    this.label = label;
  }
}
