import processing.serial.*;

Serial myPort;  // Create object from Serial class
static String val;    // Data received from the serial port
int sensorFreq = 0; // Frequency of the sound
int[] freqHistory; // Array to store frequency history
int historyLength = 500; // Length of history to display
int graphWidth = 800; // Width of the graph
int graphHeight = 400; // Height of the graph

void settings() {
  size(graphWidth, graphHeight);
}

void setup() {
  background(0);
  stroke(255);
  noFill();
  freqHistory = new int[historyLength];
  String portName = "COM11"; // Change the port name to match your setup
  myPort = new Serial(this, portName, 9600);
}

void draw() {
  if (myPort.available() > 0) {  // If data is available from the serial port
    val = myPort.readStringUntil('\n'); 
    try {
      if (val != null) { // Check if val is not null before using it
        sensorFreq = Integer.valueOf(val.trim());
        updateHistory(sensorFreq);
        drawGraph();
      }
    } catch(Exception e) {
      println("Error reading serial data: " + e);
    }
  }  
}

void updateHistory(int newFreq) {
  // Shift the frequency history array to the left
  for (int i = 0; i < historyLength - 1; i++) {
    freqHistory[i] = freqHistory[i + 1];
  }
  // Add the new frequency to the end of the history array
  freqHistory[historyLength - 1] = newFreq;
}

void drawGraph() {
  // Clear the background
  background(0);
  // Draw the graph lines
  for (int i = 0; i < historyLength - 1; i++) {
    float x1 = map(i, 0, historyLength - 1, 0, width);
    float x2 = map(i + 1, 0, historyLength - 1, 0, width);
    float y1 = map(freqHistory[i], 0, 1023, height, 0);
    float y2 = map(freqHistory[i + 1], 0, 1023, height, 0);
    line(x1, y1, x2, y2);
  }
}
